#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import datetime
from logging import getLogger
from os import path
from uuid import uuid4

from django.conf import settings
from django.utils.translation import ugettext as _
from suds import WebFault
from suds.client import Client


logger = getLogger('payline')


class PaylineProcessor(object):
    """Payline Payment Backend."""

    def __init__(self):
        """Instantiate suds client."""
        here = path.abspath(path.dirname(__file__))
        self.wsdl = getattr(
            settings, 'PAYLINE_WSDL',
            'file://%s' % path.join(here, 'DirectPaymentAPI.wsdl'))
        self.merchant_id = getattr(settings, 'PAYLINE_MERCHANT_ID', '')
        self.api_key = getattr(settings, 'PAYLINE_KEY', '')
        self.vad_number = getattr(settings, 'PAYLINE_VADNBR', '')
        self.client = Client(url=self.wsdl,
                             username=self.merchant_id,
                             password=self.api_key)

    def create_update_wallet(self, wallet_id, last_name, first_name,
                             card_number, card_type, card_expiry, card_cvx,
                             create=True):
        """Create or update a customer wallet to hold payment information.

        Return True if the creation or update was successful.

        """
        wallet = self.client.factory.create('ns1:wallet')
        wallet.walletId = wallet_id
        wallet.lastName = last_name
        wallet.firstName = first_name
        wallet.card = self.client.factory.create('ns1:card')
        wallet.card.number = card_number
        wallet.card.type = card_type
        wallet.card.expirationDate = card_expiry
        wallet.card.cvx = card_cvx
        service = self.client.service.createWallet
        if not create:
            service = self.client.service.updateWallet
        try:
            res = service(contractNumber=self.vad_number, wallet=wallet)
        except:
            logger.error("Payment backend failure", exc_info=True)
            return (False,
                    _("Payment backend failure, please try again later."))
        return (res.result.code == "02500",  # success ?
                res.result.shortMessage + ': ' + res.result.longMessage)

    def get_wallet(self, wallet_id):
        """Get wallet information from Payline."""
        try:
            res = self.client.service.getWallet(
                contractNumber=self.vad_number,
                walletId=wallet_id)
        except WebFault:
            logger.error("Payment backend failure", exc_info=True)
            return (False,
                    _("Payment backend failure, please try again later."))
        return (res.result.code == "02500",  # success ?
                getattr(res, 'wallet', None),  # None is needed because of suds
                res.result.shortMessage + ': ' + res.result.longMessage)

    def make_wallet_payment(self, wallet_id, amount):
        """Make a payment from the given wallet."""
        amount_cents = amount * 100  # use the smallest unit possible (cents)
        payment = self.client.factory.create('ns1:payment')
        payment.amount = amount_cents
        payment.currency = 978  # euros
        payment.action = 101  # authorization + validation = payment
        payment.mode = 'CPT'  # CPT = comptant
        payment.contractNumber = self.vad_number
        order = self.client.factory.create('ns1:order')
        order.ref = str(uuid4())
        order.amount = amount_cents
        order.currency = 978
        order.date = datetime.now().strftime("%d/%m/%Y %H:%M")
        try:
            res = self.client.service.doImmediateWalletPayment(
                payment=payment,
                order=order,
                walletId=wallet_id)
        except WebFault:
            logger.error("Payment backend failure", exc_info=True)
            return (False, None,
                    _("Payment backend failure, please try again later."))
        return (res.result.code == "00000",  # success ?
                res.transaction.id,
                res.result.shortMessage + ': ' + res.result.longMessage)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from datetime import datetime

from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import Wallet, get_uuid4, expiry_date_to_datetime
from .processor import PaylineProcessor


def validate_expiry_date(expiry_date):
    """Validate that the expiry date is valid."""
    try:
        exp = expiry_date_to_datetime(expiry_date)
    except ValueError:
        raise forms.ValidationError(_("Expiry date format must be MMYY"))
    if exp < datetime.today():
        raise forms.ValidationError(_("Expiry date must be in the future"))
    return exp


def obfuscate_card_number(card_number):
    """Obfuscate everything but the last four chars from a card number."""
    length = len(card_number)
    obfuscated = 'X' * (length - 4)
    return obfuscated + card_number[-4:]


class WalletForm(forms.ModelForm):
    """Create or update a wallet."""
    card_cvx = forms.CharField(label=_('Card CVX code'),
                               help_text=_('Security code, three numbers from '
                                           'the back of the payment card'),
                               max_length=3)

    class Meta:
        model = Wallet

    def __init__(self, *args, **kwargs):
        super(WalletForm, self).__init__(*args, **kwargs)
        self.pp = PaylineProcessor()
        self.create = True
        self.wallet_id = get_uuid4()

    def clean_card_expiry(self):
        expiry_date = self.cleaned_data['card_expiry']
        validate_expiry_date(expiry_date)
        return expiry_date

    def clean(self):
        """Validate that the card is correct by creating a Payline wallet."""
        cleaned_data = super(WalletForm, self).clean()
        # do not attempt a creation or update of wallet unless form is valid
        if self.errors:
            return cleaned_data
        result, message = self.pp.create_update_wallet(
            wallet_id=self.wallet_id,
            last_name=cleaned_data.get('last_name'),
            first_name=cleaned_data.get('first_name'),
            card_number=cleaned_data.get('card_number'),
            card_type=cleaned_data.get('card_type'),
            card_expiry=cleaned_data.get('card_expiry'),
            card_cvx=cleaned_data.get('card_cvx'),
            create=self.create)
        if not result:
            raise forms.ValidationError(message)
        return cleaned_data

    def save(self, commit=True):
        """Create wallet on Payline."""
        wallet = super(WalletForm, self).save(commit=commit)
        wallet.wallet_id = self.wallet_id
        wallet.card_number = obfuscate_card_number(wallet.card_number)
        if commit:
            wallet.save()
        return wallet


class UpdateWalletForm(WalletForm):

    def __init__(self, *args, **kwargs):
        super(UpdateWalletForm, self).__init__(*args, **kwargs)
        self.create = False
        self.wallet_id = self.instance.wallet_id
        self.initial['card_number'] = ''
        self.initial['card_type'] = None
        self.initial['card_expiry'] = ''
        self.initial['card_cvx'] = ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.core.urlresolvers import reverse_lazy
from django.views.generic import CreateView, UpdateView, DetailView

from .forms import WalletForm, UpdateWalletForm
from .models import Wallet


class ViewWallet(DetailView):
    model = Wallet


class UpdateWallet(UpdateView):
    form_class = UpdateWalletForm
    model = Wallet


class CreateWallet(CreateView):
    form_class = WalletForm
    model = Wallet
    success_url = reverse_lazy('view_wallet')
